/*!\file:  SpawnCoreSerial.cpp
 * \brief: run core ISSM solution using Dakota inputs. Call the Serial core solution, using mexCallMATLAB
 * \sa SpawnCore.cpp SpawnCoreParallel.cpp
 *
 */ 

#ifdef HAVE_CONFIG_H
	#include <config.h>
#else
#error "Cannot compile with HAVE_CONFIG_H symbol! run configure first!"
#endif

#include "../../objects/objects.h"
#include "../../io/io.h"
#include "../../EnumDefinitions/EnumDefinitions.h"
#include "../../shared/shared.h"
#include "./Dakotax.h"
#include "../../include/include.h"

void SpawnCoreSerial(double* responses, int numresponses, double* variables, char** variables_descriptors,int numvariables, mxArray* femmodel, int counter){

	int i;

	//inputs to matlab routine
	mxArray* mxvariables=NULL;
	double*  variables_copy=NULL;
	mxArray* mxvariabledescriptors=NULL;
	mxArray* mxcounter=NULL;
	mwSize   dims[2]={0};

	//mexCallMATLAB arrays
	mxArray* array[4];

	//output from SpawnCore in matlab routine.
	mxArray* mxresponses=NULL;

	/*Create variables and variable descriptors mxArrays that we will feed to the core solution for update of the inputs: */
	mxvariables=mxCreateDoubleMatrix(numvariables,1,mxREAL);
	variables_copy=(double*)xmalloc(numvariables*sizeof(double));
	memcpy(variables_copy,variables,numvariables*sizeof(double));
	mxSetPr(mxvariables,variables_copy);

	dims[0]=numvariables;
	dims[1]=1;
	mxvariabledescriptors=mxCreateCellArray(2,dims);
	for(i=0;i<numvariables;i++){
		mxSetCell(mxvariabledescriptors,i,mxCreateString(variables_descriptors[i]));
	}

	mxcounter=mxCreateDoubleScalar((double)counter);

	//call SpwanCore matlab routine.
	array[0]=femmodel;
	array[1]=mxvariables;
	array[2]=mxvariabledescriptors;
	array[3]=mxcounter;

	mexCallMATLAB(1,&mxresponses,4,array,"SpawnCore");

	/*copy responses back to dakota: */
	memcpy(responses,mxGetPr(mxresponses),numresponses*sizeof(double));

	//destroy constructed arrays: 
	mxDestroyArray(mxvariables);
	mxDestroyArray(mxvariabledescriptors);
	mxDestroyArray(mxresponses);
	mxDestroyArray(mxcounter);

}
